package com.cyb3rko.flashdim

import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.content.SharedPreferences.OnSharedPreferenceChangeListener
import android.os.Bundle
import android.os.VibratorManager
import android.view.Gravity
import android.widget.LinearLayout
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.res.ResourcesCompat
import androidx.preference.Preference
import androidx.preference.PreferenceFragmentCompat
import androidx.preference.PreferenceManager
import com.cyb3rko.flashdim.databinding.ActivitySettingsBinding
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import com.google.android.material.slider.Slider

internal class SettingsActivity : AppCompatActivity(), OnSharedPreferenceChangeListener {
    private lateinit var binding: ActivitySettingsBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivitySettingsBinding.inflate(layoutInflater)
        setContentView(binding.root)
        supportFragmentManager
            .beginTransaction()
            .replace(binding.settingsContainer.id, SettingsFragment())
            .commit()

        setSupportActionBar(binding.topAppBar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        PreferenceManager.setDefaultValues(this, R.xml.preferences, false)
        PreferenceManager.getDefaultSharedPreferences(this)
            .registerOnSharedPreferenceChangeListener(this)
    }

    override fun onSharedPreferenceChanged(sharedPreferences: SharedPreferences?, key: String?) {
        when (key) {
            Safe.MORSE_VIBRATION, Safe.BUTTON_VIBRATION -> {
                MaterialAlertDialogBuilder(
                    this,
                    com.google.android.material.R.style.ThemeOverlay_Material3_MaterialAlertDialog_Centered
                )
                    .setIcon(ResourcesCompat.getDrawable(
                        resources,
                        android.R.drawable.stat_notify_sync,
                        theme
                    ))
                    .setTitle(getString(R.string.dialog_restart_title))
                    .setMessage(getString(R.string.dialog_restart_message))
                    .setPositiveButton(getString(R.string.dialog_restart_positive_button)) { _, _ ->
                        val intent = packageManager.getLaunchIntentForPackage(packageName)
                        val componentName = intent!!.component
                        val mainIntent = Intent.makeRestartActivityTask(componentName)
                        startActivity(mainIntent)
                        Runtime.getRuntime().exit(0)
                    }
                    .setNegativeButton(getString(R.string.dialog_restart_negative_button), null)
                    .show()
            }
        }
    }

    internal class SettingsFragment : PreferenceFragmentCompat() {
        private lateinit var myContext: Context
        private val vibrator by lazy {
            (myContext.getSystemService(Context.VIBRATOR_MANAGER_SERVICE)
                    as VibratorManager).defaultVibrator
        }

        override fun onCreatePreferences(savedInstanceState: Bundle?, rootKey: String?) {
            myContext = requireContext()
            setPreferencesFromResource(R.xml.preferences, rootKey)

            findPreference<Preference>(Safe.INITIAL_LEVEL)?.apply {
                if (!Safe.getBoolean(myContext, Safe.MULTILEVEL, false)) {
                    isEnabled = false
                    return@apply
                }

                val initialLevel = Safe.getInt(myContext, Safe.INITIAL_LEVEL, 1)
                val summaryString = getString(R.string.preference_item_initial_level_summary)
                summary = "$summaryString: $initialLevel"

                setOnPreferenceClickListener { preference ->
                    val currentInitialLevel = Safe.getInt(myContext, Safe.INITIAL_LEVEL, 1)
                    val withVibration = Safe.getBoolean(myContext, Safe.BUTTON_VIBRATION, true)
                    val content = LinearLayout(myContext).apply {
                        orientation = LinearLayout.VERTICAL
                        setPadding(75, 0, 75, 0)
                    }
                    val levelView = TextView(myContext).apply {
                        textSize = 18f
                        setPadding(24, 24, 24, 50)
                        gravity = Gravity.CENTER_HORIZONTAL
                        text = String.format(
                            getString(R.string.preference_item_initial_level_dialog_message),
                            currentInitialLevel,
                            currentInitialLevel
                        )
                    }
                    content.addView(levelView)
                    val slider = Slider(myContext).apply {
                        valueFrom = 1F
                        valueTo = Safe.getInt(myContext, Safe.MAX_LEVEL, 0).toFloat()
                        value = currentInitialLevel.toFloat()
                        stepSize = 1F
                        addOnChangeListener { _, value, _ ->
                            if (withVibration) Vibrator.vibrateTick(vibrator)
                            levelView.text = String.format(
                                getString(R.string.preference_item_initial_level_dialog_message),
                                value.toInt(),
                                currentInitialLevel
                            )
                        }
                    }
                    content.addView(slider)

                    MaterialAlertDialogBuilder(
                        requireContext(),
                        com.google.android.material.R.style.ThemeOverlay_Material3_MaterialAlertDialog_Centered
                    )
                        .setIcon(ResourcesCompat.getDrawable(
                            resources,
                            R.drawable._ic_level,
                            requireContext().theme
                        ))
                        .setTitle(getString(R.string.preference_item_initial_level_dialog_title))
                        .setView(content)
                        .setPositiveButton(android.R.string.ok) { _, _ ->
                            val value = slider.value.toInt()
                            Safe.writeInt(myContext, Safe.INITIAL_LEVEL, value)

                            val summary = getString(R.string.preference_item_initial_level_summary)
                            preference.summary = "$summary: $value"
                        }
                        .setNegativeButton(getString(
                            R.string.preference_item_initial_level_dialog_negative_button
                        ), null)
                        .show()

                    true
                }
            }
        }
    }
}
